(***********************************************************************
This file was generated automatically by the Mathematica front end.
It contains Initialization cells from a Notebook file, which typically
will have the same name as this file except ending in ".nb" instead of
".m".

This file is intended to be loaded into the Mathematica kernel using
the package loading commands Get or Needs.  Doing so is equivalent to
using the Evaluate Initialiation Cells menu command in the front end.

DO NOT EDIT THIS FILE.  This entire file is regenerated automatically 
each time the parent Notebook file is saved in the Mathematica front end.
Any changes you make to this file will be overwritten.
***********************************************************************)

(* $Id: StatUtilities.m,v 3.0 1996/11/23 *)


(*:Version: Mathematica 3.0 *)

(*:Name: StatUtilities *)

(*:Title: Statistical Utilities Package *)


(*:Legal:
  Copyright (c) 1987-1996, David A. Belsley
*)

(*:Summary: Gaussian, SphericalStandardGaussian, MultivariateGaussian, FStat, Mean, Median, Variance, Standard Deviation, Skewness, Kurtosis, Correlation, CovarianceToCorrelationMatrix, CorrelationMatrix, EuclideanNorm, Normalized, Centered, SymmetricQ, SquareQ, LogEscale, Ecale, Truncate *)

(*:Keywords: Econometrics, statistical testing, data generation *)

(*:Requirements: No special system requirements. *)

(*:Warning: Experimental Package *)

(*:Sources: Econometrics texts, Principles of Econometrics, Wiley (1971) by H. Theil, and Conditioning Diagnostics: Collinearity and Weak Data in Regression Wiley (1991) by David A. Belsley. *)



BeginPackage["StatUtilities`"]

Gaussian::usage =
"Gaussian[mean_:0, var_:1] returns a gaussian (normal)
random variable."

SphericalStandardGaussian::usage =
"SphericalStandardGaussian[size_,seriesLength_:1] returns
a matrix of seriesLength, size-variate standard normals."

MultivariateGaussian::usage =
"MultivariateGaussian[size_,seriesLength_:1,mean_:Null,
	variance_:Identity] returns a matrix of seriesLength, 
	size-variate normals with mean-vector mean (default zero)
	and variance-covariance matrix variance (default Identity)."

Fstat::usage = 
"Fstat[estBetas_,VarCovMatrix_,hypothValues_:Null] calculates
the F test of Ho: beta = hypothValues vs. H1: beta
!= hypothValues.  estBetas can be the full set of
regression estimates; VarCovMatrix would then be sig2*xx
from the regression output. hypothValues is a list. The
dimensions of these arguments must be p, pxp, and p,
respectively.  If an element of hypothValues is a number,
that becomes the hypothesized value for that coefficient
in the test.  If an element is a non-number, such as an
NA, that coefficient is ignored in the test (i.e., the
test is marginal with respect to that coefficient). If
hypothValues is omitted, Fstat assumes the first
coefficient is a constant term and tests that all slope
coefficients are zero."

Mean::usage =
"Mean[list_List] returns the arithmetic mean of list."

Median::usage =
"Median[list_List] returns the median value of a list"

Variance::usage =
"Variance[list_List] returns the sample variance of list."

StandardDeviation::usage =
"StandardDeviation[list_List] returns the sample standard 
deviation of list."

Skewness::usage =
"Skewness[list_List] returns the alpha 3 measure of skewness
of data in list."

Kurtosis::usage =
"Kurtosis[list_List] returns the alpha 4 measure of the kurtosis
of data in list."

Correlation::usage =
"Correlation[x1_List,x2_List] returns the sample correlation
between two equal-length lists."

CovarianceToCorrelationMatrix::usage =
"CovarianceToCorrelationMatrix[CovMat_] returns the correlation
matrix corresponding to the symmetric, positive-definite
covariance matrix CovMat."

CorrelationMatrix::usage =
"CorrelationMatrix[mat_] returns the matrix of
correlations between the various rows of mat."

EuclideanNorm::usage =
"EuclideanNorm[list_] returns the Euclidean length of list."

Normalized::usage =
"Normalized[list_List] returns list normalized to have
unit Euclidean length."

Centered::usage =
"Centered[list_,centerValue_:Infinity] returns mean centered
list if centerValue is absent, otherwise returns list
centered about centerValue."

SymmetricQ::usage =
"SymmetricQ[mat_] returns True if mat is a symmetric matrix,
otherwise False."

SquareQ::usage =
"SquareQ[mat_] returns True if mat is square, otherwise False."

LogEscale::usage =
"LogEscale[list_] returns the log of the e-scaled list, i.e.,
list has not yet been logged.  Use Escale[] to e-scale an
already logged variate."

Escale::usage =
"Escale[logList_] returns the e-scaled logList, i.e., the
variate has already been logged.  Use LogEscale[] to 
e-scale and log an unlogged variate."

Truncate::usage =
"Truncate[x_, n_] returns x truncated at n digits."

Grad::usage =
"Grad[f_, x_] returns the gradient of f[] with respect
to a (sub)set of its arguments x"

Hessian::usage =
"Hessian[f_, x_] returns the Hessian matrix (matrix of
second partials) of the f[] with respect to a (sub)set of
its arguments x"

Begin["`private`"]

Gaussian[mean_:0, variance_:1] :=  N[mean + Sqrt[variance]*
	 Sqrt[-2 Log[Random[]]] Cos[2Pi Random[]]]

pi2 = N[2 Pi];
gauss = Compile[{},Sqrt[-2 Log[Random[]]] Cos[pi2 Random[]]];

SphericalStandardGaussian[size_,seriesLength_:1]:=
	If[seriesLength>1,
		Return[Table[gauss[],{size},{seriesLength}]],
		Return[Table[gauss[],{size}]]
	]

MultivariateGaussian[size_,seriesLength_:1,mean_:Null,
	variance_:Null] :=
	Block[{mu,VCM},
		If[TrueQ[mean==Null],mu = Table[0.0,{size}], mu = mean];
		If[TrueQ[variance == Null], 
			VCM = IdentityMatrix[size],
			If[!SymmetricQ[variance],
				Message[MultivariateGaussian::nSqu];
				Return[]
			];
			If[Length[mu]==Dimensions[variance][[1]],
				VCM = SingularValues[variance//N];
				VCM = Transpose[VCM[[1]]].DiagonalMatrix[Sqrt[VCM[[2]]]],
				Message[MultivariateGaussian::size];
				Return[]
			]
		];
		Return[mu+VCM.SphericalStandardGaussian[size,seriesLength]]
	]
MultivariateGaussian::nSqu := "Var/Cov matrix must be square and symmetric";
MultivariateGaussian::size := "mean and var/cov matrix not conformable in size";


Fstat[betaEstimates_,VarMat_,betaNull_:Null] := 
Block[{i,p,q,testValues={},olsBetas=betaEstimates,var = VarMat,
				betas},
	If[!SymmetricQ[var],
		Message[Fstat::nSqu];
		Return[]
	];
	{p,q} = Dimensions[var];
	If[TrueQ[betaNull == Null],
		betas = Prepend[Table[0.0,{p-1}],"a"],
		betas = betaNull
	];
	If[Length[betas]!=p || Length[betaEstimates]!=p,
		Message[Fstat::size];
		Return[]
	];
	For[i=p,i>=1,i--,
		If[NumberQ[betas[[i]]],
			PrependTo[testValues,betas[[i]]//N],
			q--;
			olsBetas = Drop[olsBetas,{i,i}];
			var = Drop[Transpose[Drop[var,{i,i}]],{i,i}];
		]
	];
	testValues = testValues-olsBetas;
	Return[(testValues.PseudoInverse[var].testValues)/q]
]
Fstat::nSqu := "Variance-Covariance matrix must be square and symmetric";
Fstat::size := "sizes of coefs and VarCov are incompatible";


Mean[list_] := With[{n = Length[list]},
					(Table[1.,{n}].list)/n]

Median[list_] := 
Block[{n = Length[list]},
	If[OddQ[n], Sort[list][[(n+1)/2]], 
		Apply[Plus,Take[Sort[list],{n/2,n/2+1}]]/2.
	]
]

Variance[list_List] := 
Block[{temp = Centered[list]},
	N[1.0/(Length[list]-1.0) temp.temp]
]
	
StandardDeviation[list_List] := Sqrt[Variance[list]]

Skewness[list_List] :=
Block[{temp = Centered[list],n = Length[list],temp2},
	If[(temp2 = temp.temp)==0.,Indeterminate,
		(((n-1)^(1.5))/n)((temp^2).temp)/Sqrt[temp2]^3
	]
]

Kurtosis[list_List] :=
Block[{temp = Centered[list],n = Length[list],temp2},
	If[(temp2 = temp.temp)==0.,Indeterminate,
		(((n-1)^2.)/n)((temp^2).(temp^2))/temp2^2
	]
]

Correlation[x1_List,x2_List] :=
	Block[{Z = N[{x1,x2}]},
		Z = Z-Mean[Transpose[Z]];
		Z = Z.Transpose[Z];
		If[Z[[1,1]] == 0. || Z[[2,2]] == 0.,Indeterminate,
		Z[[1,2]]/Sqrt[Z[[1,1]] Z[[2,2]]]
		]
	]/;Length[x1] == Length[x2]

CovarianceToCorrelationMatrix[CovMat_] :=
	Block[{Diag},
		Diag = Sqrt[DiagonalMatrix
					[1.0/Map[CovMat[[#,#]]&,Range[Length[CovMat]]]]];
		Diag.CovMat.Diag
	]/;SymmetricQ[CovMat]

CorrelationMatrix[mat_] :=
	Block[{Z},
		Z = mat-Mean[Transpose[mat]];
		Z = Z.Transpose[Z];
		CovarianceToCorrelationMatrix[Z]
	]

Normalized[list_List] := list/EuclideanNorm[list]

EuclideanNorm[list_] := N[Sqrt[list.list]]

SymmetricQ[mat_] := TrueQ[MatrixQ[mat] && mat == Transpose[mat]]

SquareQ[mat_] := TrueQ[MatrixQ[mat] && Apply[Equal,Dimensions[mat]]]

Centered[list_,centerValue_:Infinity] := 
	If[TrueQ[centerValue === Infinity],
		Return[list - Mean[list]],
		Return[list - centerValue]
	]//N

LogEscale[z_] := N[Centered[Log[z]] + 1.]

Escale[lnz_] := N[Centered[lnz] + 1.]

Grad[f_,x_] := Map[D[f,#]&,x]
Hessian[f_,x_] := Grad[Grad[f,x],x]

Truncate[x_, n_] := N[Return[Round[x*10.^n]/10.^n]]

End[]

EndPackage[]